@@ -3,6 +3,12 @@
 #ifdef DEFAULT_SIGNATURES
 {-# LANGUAGE DefaultSignatures #-}
 #endif
+{-# LANGUAGE CPP #-}
+#if __GLASGOW_HASKELL__ >=704
+{-# LANGUAGE Safe #-}
+#elif __GLASGOW_HASKELL__ >=702
+{-# LANGUAGE Trustworthy #-}
+#endif
 -- | Bottoms are ignored for this entire module:
 -- only fully-defined inhabitants are considered inhabitants.
 module Data.Universe.Class
@@ -37,8 +43,20 @@ import qualified Data.Semigroup as Semi
 import qualified Data.Set as Set
 import qualified Data.Map as Map
 
+#if MIN_VERSION_base(4,16,0)
+import Data.Tuple (Solo (..))
+#elif MIN_VERSION_base(4,15,0)
+import GHC.Tuple (Solo (..))
+#else
+import Data.Tuple.Solo (Solo (..))
+#endif
+
 -- $setup
 -- >>> import Data.List
+-- >>> import Data.Universe.Helpers
+--
+-- -- Show (a -> b) instance (in universe-reverse-instances, but cannot depend on it here).
+-- >>> instance (Finite a, Show a, Show b) => Show (a -> b) where showsPrec n f = showsPrec n [(a, f a) | a <- universeF]
 
 -- | Creating an instance of this class is a declaration that your type is
 -- recursively enumerable (and that 'universe' is that enumeration). In
@@ -221,9 +239,16 @@ instance RationalUniverse Natural where
 --
 -------------------------------------------------------------------------------
 
--- could change the Ord constraint to an Eq one, but come on, how many finite
--- types can't be ordered?
+-- |
+-- >>> mapM_ print (universe :: [Bool -> Bool])
+-- [(False,False),(True,False)]
+-- [(False,False),(True,True)]
+-- [(False,True),(True,False)]
+-- [(False,True),(True,True)]
+--
 instance (Finite a, Ord a, Universe b) => Universe (a -> b) where
+  -- could change the Ord constraint to an Eq one, but come on, how many finite
+  -- types can't be ordered?
   universe = map tableToFunction tables where
     tables          = choices [universe | _ <- monoUniverse]
     tableToFunction = (!) . fromList . zip monoUniverse
@@ -295,6 +320,19 @@ instance Finite a => Finite (Semi.Min   a) where universeF = map Semi.Min   univ
 instance Finite a => Finite (Semi.First a) where universeF = map Semi.First universeF; cardinality = retagWith Semi.First cardinality
 instance Finite a => Finite (Semi.Last  a) where universeF = map Semi.Last  universeF; cardinality = retagWith Semi.Last  cardinality
 
+-- |
+-- >>> mapM_ print (universeF :: [Bool -> Bool])
+-- [(False,False),(True,False)]
+-- [(False,False),(True,True)]
+-- [(False,True),(True,False)]
+-- [(False,True),(True,True)]
+--
+-- >>> cardinality :: Tagged (Bool -> Ordering) Natural
+-- Tagged 9
+--
+-- >>> cardinality :: Tagged (Ordering -> Bool) Natural
+-- Tagged 8
+--
 instance (Ord a, Finite a, Finite b) => Finite (a -> b) where
   universeF = map tableToFunction tables where
     tables          = sequence [universeF | _ <- monoUniverse]
@@ -327,6 +365,14 @@ instance Finite a => Finite (Tagged b a) where cardinality = retagWith Tagged ca
 -- containers
 -------------------------------------------------------------------------------
 
+-- |
+-- >>> import qualified Data.Set as Set
+-- >>> mapM_ print (universe :: [Set.Set Bool])
+-- fromList []
+-- fromList [False]
+-- fromList [True]
+-- fromList [False,True]
+--
 instance (Ord a, Universe a) => Universe (Set.Set a) where
     universe = Set.empty : go universe
       where
@@ -337,10 +383,23 @@ instance (Ord a, Universe a) => Universe (Set.Set a) where
             inter []     = []
             inter (y:ys) = y : Set.insert x y : inter ys
 
-
 instance (Ord a, Finite a) => Finite (Set.Set a) where
     cardinality = retag (fmap (2 ^) (cardinality :: Tagged a Natural))
 
+-- |
+-- >>> import qualified Data.Map as Map
+-- >>> mapM_ print (universe :: [Map.Map Bool Bool])
+-- fromList []
+-- fromList [(True,False)]
+-- fromList [(False,False)]
+-- fromList [(True,True)]
+-- fromList [(False,False),(True,False)]
+-- fromList [(False,True)]
+-- fromList [(False,False),(True,True)]
+-- fromList [(False,True),(True,False)]
+-- fromList [(False,True),(True,True)]
+--
+--
 instance (Ord k, Finite k, Universe v) => Universe (Map.Map k v) where
   universe = map tableToFunction tables where
     tables          = choices [universe | _ <- monoUniverse]
@@ -387,3 +446,13 @@ instance (Finite (f a), Finite (g a))      => Finite   (Sum f g a) where
   cardinality = liftM2 (+)
     (retag (cardinality :: Tagged (f a) Natural))
     (retag (cardinality :: Tagged (g a) Natural))
+
+-------------------------------------------------------------------------------
+-- OneTuple
+-------------------------------------------------------------------------------
+
+-- @since 1.1.3
+instance  Universe a => Universe (Solo a) where universe  = map Solo universe
+
+-- @since 1.1.3
+instance  Finite   a => Finite   (Solo a) where universeF = map Solo universeF; cardinality = retagWith Solo cardinality
